﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Parsing
{
    // Program parsing and processing the log files containing query execution times.
    // It subtracts the times to obtain time spent between the client and the server,
    // in the middleware and in the controller.
    class Program
    {
        static double TimeToDouble(string time)
        {
            return double.Parse(time.Substring(6, 10), 
                System.Globalization.NumberStyles.AllowDecimalPoint, 
                System.Globalization.NumberFormatInfo.InvariantInfo);
        }

        static void Main(string[] args)
        {
            bool SQL_VERSION = true;
            //bool SQL_VERSION = false;

            /***** Parse log files *****/
            Dictionary<string, double> ControllerDict = 
                new Dictionary<string, double>();
            Dictionary<string, Tuple<string, double>> MiddlewareDict = 
                new Dictionary<string, Tuple<string, double>>();
            Dictionary<string, double> ClientDict = 
                new Dictionary<string, double>();

            StreamReader reader = File.OpenText("C:\\EvsLogs\\MultiMonitoring\\ext-tva.log");
            string line;
            while ((line = reader.ReadLine()) != null)
            {                
                string[] items = line.Split(' ');

                switch (items[6])
                {
                    case "JobV2Controller":
                    case "JobController":
                        {
                            // requestId    time_controller
                            double time = TimeToDouble(items[8]);
                            ControllerDict.Add(items[7], time);
                        }
                        break;
                    case "MiddleWare":
                        {
                            // requestId    trackingId     time_middleware
                            double time = TimeToDouble(items[9]);
                            MiddlewareDict.Add(items[7], new Tuple<string, double>(items[8], time));
                            break;
                        }
                    case "Client":
                        {
                            // trackingId	time_client
                            double time = TimeToDouble(items[8]);
                            ClientDict.Add(items[7], time);
                            break;
                        }

                    default:
                       Console.WriteLine("Error parsing");
                }
            }
            
            
            /***** Put results together *****/
            List<List<double>> results = new List<List<double>>();
            Dictionary<string, double>.KeyCollection ControllerDictKeys = ControllerDict.Keys;

            Console.WriteLine();
            foreach (string key in ControllerDictKeys)
            {
                double TimeController;
                if (ControllerDict.TryGetValue(key, out TimeController))
                {
                    Tuple<string, double> tmp;
                    if (MiddlewareDict.TryGetValue(key, out tmp))
                    {
                        string trackingId = tmp.Item1;
                        double MiddlewareTime = tmp.Item2;

                        double TimeClient;
                        if (ClientDict.TryGetValue(trackingId, out TimeClient))
                        {
                            results.Add(new List<double> { TimeController, MiddlewareTime, TimeClient });
                        }
                        else
                            Console.WriteLine("Key = {0} is not found in ClientDict.", trackingId);
                    }
                    else
                        Console.WriteLine("Key = {0} is not found in MiddlewareDict.", key);
                }
                else
                {
                    Console.WriteLine("Key = {0} is not found in ControllerDict.", key);
                }
            }


            List<double> SQLTime = new List<double>();
            if (SQL_VERSION)
            {
                // SQL query execution time
                StreamReader readerSQL = File.OpenText("C:\\EvsLogs\\MultiMonitoring\\sql.log");
                while ((line = readerSQL.ReadLine()) != null)
                {
                    Console.WriteLine(Convert.ToDouble(line) / 1000000);
                    SQLTime.Add(Convert.ToDouble(line) / 1000000);
                }
                Console.WriteLine("Number of SQL query time measures: " + SQLTime.Count);
            }


            /***** Display results *****/

            // Write the string to a file.
            string path = "";
            string pathCSVCumulative = "";
            string pathCSVSeparated = "";
            if (SQL_VERSION)
            {
                path = "C:\\Users\\ext-tva\\Desktop\\tfe\\Tests\\sql_results.txt";
                pathCSVCumulative = "C:\\Users\\ext-tva\\Desktop\\tfe\\Tests\\sql_results_cumul.csv";
                pathCSVSeparated = "C:\\Users\\ext-tva\\Desktop\\tfe\\Tests\\sql_results_separate.csv";
            }
            else
            {
                path = "C:\\Users\\ext-tva\\Desktop\\tfe\\Tests\\cache_results.txt";
                pathCSVCumulative = "C:\\Users\\ext-tva\\Desktop\\tfe\\Tests\\cache_results_cumul.csv";
                pathCSVSeparated = "C:\\Users\\ext-tva\\Desktop\\tfe\\Tests\\cache_results_separate.csv";
            }

            System.IO.StreamWriter file = new System.IO.StreamWriter(path);
            System.IO.StreamWriter fileCSVCumulative = new System.IO.StreamWriter(pathCSVCumulative);
            System.IO.StreamWriter fileCSVSeparated = new System.IO.StreamWriter(pathCSVSeparated);

            file.WriteLine("Request: "); // fill in manually
            file.WriteLine("# of jobs: "); // fill in manually
            file.WriteLine();

            file.WriteLine("Temps cumulés:");
            file.WriteLine();
            double sumController = 0;
            double sumMiddleWare = 0;
            double sumClient = 0;
            int i = 0;
            
            foreach (List<double> result in results)
            {
                sumController += result[0];
                sumMiddleWare += result[1];
                sumClient += result[2];
                if (SQL_VERSION)
                {
                    file.WriteLine(SQLTime[i] + "   \t{0} ({1}%)\t{2} ({3}%)\t{4} ({5}%)",
                        result[0], (int)(100 * result[0] / result[2]),
                        result[1], (int)(100 * result[1] / result[2]),
                        result[2], (int)(100 * result[2] / result[2]));

                    fileCSVCumulative.WriteLine("{0};{1};{2};{3}",
                        SQLTime[i],
                        result[0],
                        result[1],
                        result[2]);
                }
                else
                {
                    file.WriteLine("{0} ({1}%)\t{2} ({3}%)\t{4} ({5}%)",
                         result[0], (int)(100 * result[0] / result[2]),
                         result[1], (int)(100 * result[1] / result[2]),
                         result[2], (int)(100 * result[2] / result[2]));

                    fileCSVCumulative.WriteLine("{0};{1};{2}",
                        result[0],
                        result[1],
                        result[2]);         
                }
                i++;
            }

            file.WriteLine();

            file.WriteLine("Temps séparés:");
            file.WriteLine();
            i = 0;
            
            foreach (List<double> result in results)
            {
                var r0 = result[0];
                var r1 = result[1];
                var r2 = result[2];

                if (SQL_VERSION)
                {
                    var rs = SQLTime[i];

                    file.WriteLine(rs + " (" + (int)(100 * rs / result[2]) + "%)       \t{0} ({1}%)\t{2} ({3}%)\t{4} ({5}%)",
                        r0 - rs, (int)(100 * (r0 - rs) / result[2]),
                        r1 - r0, (int)(100 * (r1 - r0) / result[2]),
                        r2 - r1, (int)(100 * (r2 - r1) / result[2]));

                    fileCSVSeparated.WriteLine("{0};{1};{2};{3}",
                        rs,
                        r0 - rs,
                        r1 - r0,
                        r2 - r1);
                }
                else
                {
                    file.WriteLine("{0} ({1}%)\t{2} ({3}%)\t{4} ({5}%)",
                        r0, (int)(100 * r0 / result[2]),
                        r1 - r0, (int)(100 * (r1 - r0) / result[2]),
                        r2 - r1, (int)(100 * (r2 - r1) / result[2]));

                    fileCSVSeparated.WriteLine("{0};{1};{2}",
                        r0,
                        r1 - r0,
                        r2 - r1);   
                }
                i++;
            }

            // means
            int count = results.Count;
            file.WriteLine();
            file.WriteLine("Mean of controller time: \t" + sumController / count);
            file.WriteLine("Mean of middleware time: \t" + sumMiddleWare / count);
            file.WriteLine("Mean of client time: \t\t" + sumClient / count);


            file.Close();
            fileCSVCumulative.Close();
            fileCSVSeparated.Close();
        }
    }
}
